Package: genomation
Type: Package
Title: Summary, annotation and visualization of genomic data
Version: 1.42.0
Author: Altuna Akalin [aut, cre], Vedran Franke [aut, cre], Katarzyna Wreczycka
    [aut], Alexander Gosdschan [ctb], Liz Ing-Simmons [ctb], Bozena Mika-Gospodorz [ctb]
Maintainer: Altuna Akalin <aakalin@gmail.com>, Vedran Franke
 <vedran.franke@gmail.com>, Katarzyna Wreczycka <katwre@gmail.com>
Description: A package for summary and annotation of genomic intervals. Users
    can visualize and quantify genomic intervals over pre-defined functional
    regions, such as promoters, exons, introns, etc. The genomic intervals
    represent regions with a defined chromosome position, which may be associated
    with a score, such as aligned reads from HT-seq experiments, TF binding sites,
    methylation scores, etc. The package can use any tabular genomic feature data
    as long as it has minimal information on the locations of genomic intervals. In
    addition, It can use BAM or BigWig files as input.
License: Artistic-2.0
LazyLoad: yes
VignetteBuilder: knitr
biocViews: Annotation, Sequencing, Visualization, CpGIsland
Encoding: latin1
URL: http://bioinformatics.mdc-berlin.de/genomation/
BugReports: https://github.com/BIMSBbioinfo/genomation/issues
Depends: R (>= 3.5.0), grid
Imports: Biostrings (>= 2.47.6), BSgenome (>= 1.47.3), data.table,
        Seqinfo, GenomicRanges (>= 1.31.8), GenomicAlignments (>=
        1.15.6), S4Vectors (>= 0.17.25), ggplot2, gridBase, impute,
        IRanges (>= 2.13.12), matrixStats, methods, parallel, plotrix,
        plyr, readr, reshape2, Rsamtools (>= 1.31.2), seqPattern,
        rtracklayer (>= 1.39.7), Rcpp (>= 0.12.14)
Suggests: BiocGenerics, genomationData, knitr, RColorBrewer, rmarkdown,
        RUnit
Collate: 'combineScoreMatrixList.R' 'documentData.R'
        'genomation-classes.R' 'getRandomEnrichment.R'
        'findFeatureComb.R' 'patternMatrix.R' 'plotMatrix.R'
        'randomizeFeature.R' 'readAnnotate.R' 'readData.R'
        'scoreMatrix.R' 'scoreMatrixBin.R' 'scoreMatrixList.R' 'Ops.R'
        'test_genomation_package.R' 'deprecated_defunct.R'
        'enrichmentMatrix.R' 'RcppExports.R'
LinkingTo: Rcpp
RoxygenNote: 6.0.1.9000
git_url: https://git.bioconductor.org/packages/genomation
git_branch: RELEASE_3_22
git_last_commit: 01d23ac
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 04:08:16 UTC; biocbuild
