# documents data sets that come with the package

#' Example promoter data set.
#' 
#' promoters of hg19 assembly of human genome on chr21 and chr22. Promoter set is 
#' derived from refseq TSS.
#' @name promoters
#' @docType data
#' @format  \code{\link{GRanges}} object
#'
NULL


#' Example CpG island data set.
#' 
#' CpG islands of hg19 assembly of human genome on chr21 and chr22. Downloaded
#' from UCSC genome browser.
#' 
#' @name cpgi
#' @docType data
#' @format  \code{\link{GRanges}} object
#'
NULL



#' Example RefSeq genes data set.
#' 
#' RefSeq genes of hg19 assembly of human genome on chr21 and chr22. Downloaded
#' from UCSC genome browser.
#' 
#' @name genes
#' @docType data
#' @format  \code{\link{GRanges}} object
#'
NULL



#' Example CAGE data set.
#' 
#' Location and tag per million values for CAGE TSS clusters on chr21 and chr22
#' of human genome (hg19 assembly). The clusters are dowloaded from ENCODE project
#' downloads for NHEK cells.
#' 
#' @name cage
#' @docType data
#' @format  \code{\link{GRanges}} object
#'
NULL