% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{getColors}
\alias{getColors}
\title{getColors function}
\usage{
getColors(n)
}
\arguments{
\item{n}{n}
}
\description{
gets colors for a factor variable
}
\keyword{internal}
