% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{galpTo2Ranges}
\alias{galpTo2Ranges}
\title{galpTo2Ranges function}
\usage{
galpTo2Ranges(x)
}
\arguments{
\item{x}{object}
}
\description{
galpTo2Ranges function
}
\keyword{internal}
