% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{Sum_c}
\alias{Sum_c}
\title{Function that computes a sum value}
\usage{
Sum_c(x)
}
\arguments{
\item{x}{NumericVector}
}
\description{
Function that computes a sum value
}
\keyword{internal}
