% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreMatrix.R
\name{file.ext}
\alias{file.ext}
\title{file.ext function}
\usage{
file.ext(x)
}
\arguments{
\item{x}{file}
}
\description{
Extract file extension from file path
}
\keyword{internal}
