% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{Max_c}
\alias{Max_c}
\title{Function that computes a max value}
\usage{
Max_c(x)
}
\arguments{
\item{x}{NumericVector}
}
\description{
Function that computes a max value
}
\keyword{internal}
