% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readData.R
\name{read.zip}
\alias{read.zip}
\title{read.zip function}
\usage{
read.zip(file, ...)
}
\arguments{
\item{file}{zip file}

\item{...}{additional objects and parameters}
}
\description{
read.zip function
}
\keyword{internal}
